/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "project.h"

var ImgList:TImageList;

{$R symbols.res}

Constructor TDisassemblyList.Create(aOwner:TComponent);
Begin
  inherited Create(aOwner);
  DefaultRowHeight:=12;
  ColCount:=3;
  Ctl3D:=False;
  FixedCols:=0;
  FixedRows:=1;
  Font.Name:='Courrier';
  Options:=[goFixedVertLine,goColSizing,goThumbTracking];
  AdSize:=4;
  Canvas.Font.Name:='Courrier';
  Canvas.Font.Size:=10;
  IP:=1;
  //couleurs par dfaut
  clIP:=clBlue;
  clBreak:=clRed;
  clIPPen:=clWhite;
  clBreakPen:=clBlack;
End;

procedure TDisassemblyList.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);
Var Address:Longint;
  Procedure HighLightLine(Color,PColor:TColor);
  Var x:integer;
  Begin
    //HighLight a line
    Canvas.Brush.Color:=Color;
    Canvas.Pen.Color:=Color;
    Canvas.Font.Color:=PColor;
    if ACol=0 then x:=ARect.Left+20 else x:=ARect.Left;
    Canvas.Rectangle(x,ARect.Top,ARect.Right+1,ARect.Bottom);
  End;

  Procedure DrawIP;
  Begin
    if ACol=0 then ImgList.Draw(Canvas,ARect.Left,ARect.Top+2,0);
  End;

  Procedure DrawBreak;
  Begin
    if ACol=0 then ImgList.Draw(Canvas,ARect.Left+10,ARect.Top+2,1);
  End;

Begin
  if ARow=0 then
  Begin
    Case ACol of
      0:Canvas.TextOut(ARect.Left,ARect.Top,'Address');
      1:Canvas.TextOut(ARect.Left,ARect.Top,'Code');
      2:Canvas.TextOut(ARect.Left,ARect.Top,'Disassembly code');
    End;
  End else
  Begin
    Address:=Offset+ARow-1;
    if Address=IP then
    Begin
      HighLightLine(clIp,ClIpPen);
      DrawIP;
    End else
    Begin
      if Address=3 then
      HighLightLine(clBreak,clBreakPen);
    End;
    DrawBreak;
    Case Acol of
      0:Canvas.TextOut(ARect.Left+22,ARect.Top,Format('%0.'+char(AdSize+48)+'X',[Address]) );
    End;
    Canvas.Font.Color:=clBlack;
  End;
End;

begin
  ImgList:=TImageList.Create(nil);
  ImgList.Height:=8;
  ImgList.Width:=8;
  ImgList.BkColor:=clNone;
  ImgList.ResourceLoad(rtbitmap,'DEBUGSYMBOLS',$C0C0C0);
end.

