/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//---------------------------------------------------------------------------
#include "project.h"

/****************************************************************************
Classe pin_OscilloInput
****************************************************************************/
pin_OscilloInput::pin_OscilloInput(int name, component *comp, attribute *attribs, symbol *parent
  ,int memsize):pin(name,comp,attribs,parent)
{
  lowcarelevel=-INFINITY;
  highcarelevel=+INFINITY;
  pin_OscilloInput::memsize=memsize;
  buffer=new real[memsize];
  for (int i=0;i<memsize;i++) buffer[i]=0;
  min=+INFINITY;
  max=-INFINITY;
  sum=mean=0;
}

void pin_OscilloInput::_SetVin(real Vin)
{
  max=MAX(max,Vin);
  min=MIN(min,Vin);
}

real pin_OscilloInput::GetVout()
{
  return 0;
}

real pin_OscilloInput::GetZ()
{
  return 1E10;
}

pin_OscilloInput::~pin_OscilloInput()
{
  delete buffer;
}

/****************************************************************************
Classe Oscillotype
****************************************************************************/

component* Oscillotype::create(int name, attribute *attribs, symbol *parent)
{
  return new Oscillo(this,name,attribs,parent);
}

/****************************************************************************
Classe Oscillo
****************************************************************************/

Oscillo::Oscillo(componenttype *type, int name, attribute *attribs, symbol *parent)
  : component(type,name,attribs,parent),
  CLK(symbtable.toid("CLK"),this,attribs,this,(char*)&SpecialPins,0x01),
  TRIGGER(symbtable.toid("TRIGGER"),this,attribs,this,(char*)&SpecialPins,0x02)
{
  memsize=1024; //default number of points per input
  attribs->getintattribute(symbtable.toid("memsize"), &memsize);
  Inputs=(list<pin_OscilloInput>*)NULL;
  pincount=0;
  index=0;
}

void Oscillo::Update(int force)
{
  if ((SpecialPins&2)&&(!(LastSpecialPins&2))) acquire=true;
  if (acquire&&((SpecialPins&1)&&(!(LastSpecialPins&1)))) {
    for (list<pin_OscilloInput> *l = Inputs; l != NULL; l = l->next) {
      //Save pin value
      l->elem->sum+=l->elem->buffer[index]=l->elem->GetVin();
      if (index==memsize-1)
        l->elem->mean=l->elem->sum/memsize;
    }
    index++;
    if (index==memsize) {
      acquire=false;
      index=0;
    }  
  }
  LastSpecialPins=SpecialPins;
  updatetime=INFINITY;
  updateneeded=0;
}

symbol* Oscillo::_getlocalsymbol(symbolname *name)
{
  if (*symbtable.tostring(*(name->elem)) == '_') { //Seuls noms valides: _*
    int id=*(name->elem);
    for (list<pin_OscilloInput> *l = Inputs; l != NULL; l = l->next) {
      if (l->elem->getname()==id) return l->elem;
    }
    pin_OscilloInput* newpin=new pin_OscilloInput(*(name->elem), this, getattributes(), this, memsize);
    Inputs=new list<pin_OscilloInput>(newpin,Inputs);
    pincount++;
    return newpin->getlocalsymbol(name->next);
  }
  return (symbol*)NULL;
}

symbolwidget* Oscillo::getwidget()
{
  return new oscillowidget(getattributes(), this);
}

//---------------------------------------------------------------------------

