/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "project.h"

// base -> sets variable base
// signed -> non zero for signed operation

int attribute::getstringattribute(int name, char **val, int maxpos)
{ 
  int p; 
  attribute *attrib = getattribute(name, &p);
  if (maxpos == 0 || p < maxpos) 
  {
    attrib->getstringattribute(val); 
    return p;
  }
  else return 0;
}

int attribute::getintattribute(int name, int *i, int maxpos) 
{ 
  int p; 
  attribute *attrib = getattribute(name, &p);
  if (maxpos == 0 || p < maxpos) 
  {
    attrib->getintattribute(i); 
    return p;
  }
  else return 0;
}

int attribute::getnameattribute(int name, symbolname **s, int maxpos)
{ 
  int p; 
  attribute *attrib = getattribute(name, &p);
  if (maxpos == 0 || p < maxpos) 
  {
    attrib->getnameattribute(s); 
    return p;
  }
  else return 0;
}

int attribute::getrealattribute(int name, real *r, int maxpos) 
{ 
  int p; 
  attribute *attrib = getattribute(name, &p);
  if (maxpos == 0 || p < maxpos) 
  {
    attrib->getrealattribute(r); 
    return p;
  }
  else return 0;
}


attribute *_nullattribute()
{
  static attribute nattrib;

  return &nattrib;
}
  
attribute *attribute::addattribute(_attribute *newattrib)
{ 
  return newattrib; 
}

attribute *_attribute::getattribute(int name, int *pos)
{
  if (name == _attribute::name) 
  {
    if (pos != NULL) *pos = 1;
    return this;
  }

  attribute *retval = next->getattribute(name, pos);
  if (pos != NULL && *pos != 0) (*pos)++;
  return retval;
}

_attribute::_attribute(int name, attribute *next)
{
  _attribute::name = name;
  _attribute::next = next;
}

attribute *_attribute::addattribute(_attribute *newattrib)
{
  newattrib->next = this;
  return newattrib;
}

_intattribute::_intattribute(int name, int val, attribute *next) : _attribute(name, next)
{
  _intattribute::val = val;
}

_stringattribute::_stringattribute(int name, char *val, attribute *next) : _attribute(name, next)
{
  _stringattribute::val = strdup(val);
}

_realattribute::_realattribute(int name, real val, attribute *next) : _attribute(name, next)
{
  _realattribute::val = val;
}

_nameattribute::_nameattribute(int name, symbolname *val, attribute *next) : _attribute(name, next)
{
  _nameattribute::val = val;
}

_pointerattribute::_pointerattribute(int name, int attribname, attribute *next) : _attribute(name, next)
{
  _pointerattribute::attribname = attribname;
}

_emptyattribute::_emptyattribute(int name, attribute *next) : _attribute(name, next)
{
}

