/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "project.h"

list<breakpt> *breakpt::breaks = (list<breakpt> *) NULL;

void breakpt::add(breakpt *b)
{
  breaks = new list<breakpt>(b, breaks);
}

void breakpt::remove(breakpt *b)
{
  list<breakpt> **pl = &breaks;
  for (list<breakpt> *l = *pl; l != NULL; l = *pl)
  {
    if (l->elem == b)
    {
      *pl = l->next;
      l->next = (list<breakpt> *) NULL;
      delete l;
    }

    pl = &l->next;
  }
}

breakpt *breakpt::eval()
{
  for (list<breakpt> *l = breaks; l != NULL; l = l->next)
    if (l->elem->eval()) 
    {
      if (l->elem->callback == NULL ? 1 : l->elem->callback(l->elem))
	return l->elem;
    }

  return (breakpt *) NULL;
}

int breakvar::_eval()
{
  long curval = v->getsignedvalue();

  int rslt = OP_EQUAL;
  if (curval > value) rslt = OP_GREATER;
  else if (curval < value) rslt = OP_LOWER;

  return (rslt & op) != 0;
}

int breakreal::_eval()
{
  real curval = r->getvalue();

  int rslt = OP_EQUAL;
  if (curval > value) rslt = OP_GREATER;
  else if (curval < value) rslt = OP_LOWER;

  return (rslt & op) != 0;
}

char *breakop2::_tostring()
{
  char *s1 = b1->tostring();
  char *s2 = b2->tostring();
  char *s3 = getsymbol();

  char *s = new char[strlen(s1) + strlen(s2) + strlen(s3) + 5];

  strcpy(s, "(");
  strcat(s, s1);
  strcat(s, " ");
  strcat(s, s3);
  strcat(s, " ");
  strcat(s, s2);
  strcat(s, ")");

  return s;
}

char *breakop1::_tostring()
{
  char *s1 = b->tostring();
  char *s2 = getsymbol();

  char *s = new char[strlen(s1) + strlen(s2) + 1];

  strcpy(s, s2);
  strcat(s, s1);

  return s;
}

char *breakvar::_tostring()
{
  char buff[100];
  snprintf(buff, 100, "%s %s %li", v->getfullnametext(), compop(op), v->getvalue());

  char *s = new char[strlen(buff)];
  return strcpy(s, buff);
}

char *breakreal::_tostring()
{
  char buff[100];
  snprintf(buff, 100, "%s %s %e", r->getfullnametext(), compop(op), r->getvalue());

  char *s = new char[strlen(buff)];
  return strcpy(s, buff);
}
