/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __DISASSEMBLY_H__
#define __DISASSEMBLY_H__

class disassembly : public symbol
{
public:
  symbolwidget *getwidget() { return new disassemblywidget(getattributes(), this); }
  
  virtual void ToCursor(int Addr) = 0;
  virtual void ToggleBreak(int Addr) = 0;
  virtual void ClearBreakpoints() = 0;
  virtual int IsBreak(int Addr) = 0;
  virtual char *GetLine(int Addr) = 0; // Functions owns the memory.
  virtual char *GetSourceCode(int Addr) { return ""; };
  virtual int GetIP() = 0;
  virtual void SetIP(int Addr) = 0;
  virtual void GetMemRange(int *start, int *end) = 0; // Start included, end not included. If NULL, do not fill in!
  virtual char *GetOpCode(int Addr) = 0; // Returns a displayable string.
  virtual int GetOpCodeLength(int Addr) = 0;
  virtual char *GetRealAddr(int Addr) = 0;
  disassembly(int name, attribute *attribs, symbol *parent) : symbol(name, attribs, parent) {};
};

#endif
