/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __SXOPCODE_H__
#define __SXOPCODE_H__



//Logic instructions
#define AND_frW   0x160  // f5
#define AND_Wfr   0x140  // f5
#define AND_Wlit  0xE00  // k8
#define NOT_fr	  0x260  // f5
#define OR_frW	  0x120  // f5
#define OR_Wfr	  0x100  // f5
#define OR_Wlit   0xD00  // k8
#define XOR_frW	  0x1A0  // f5
#define XOR_Wfr	  0x180  // f5
#define XOR_Wlit  0xF00  // k8

//Arithmetic and Shift Instructions

#define ADD_frW	  0x1E0  // f5
#define ADD_Wfr	  0x1C0  // f5
#define CLR_fr	  0x060  // f5
#define CLR_W	  0x040  // -
#define CLR_WDT	  0x004  // -
#define DEC_fr	  0x0E0  // f5
#define DECSZ_fr  0x2E0  // f5
#define INC_fr	  0x2A0  // f5
#define INCSZ_fr  0x3E0  // f5
#define RL_fr	  0x360  // f5
#define RR_fr	  0x320  // f5
#define SUB_frW	  0x0A0  // f5
#define SWAP_fr	  0x3A0  // f5

//Bitwise Operation Instructions
#define CLRB_frb  0x400  // b3 f5
#define SB_frb	  0x700  // b3 f5
#define SETB_frb  0x500  // b3 f5
#define SNB_frb	  0x600  // b3 f5

//Data Movement Instructions
#define MOV_frW	      0x020  // f5
#define MOV_Wfr	      0x200  // f5
#define MOV_Wfr_W     0x080  // f5
#define MOV_Wlit      0xC00  // k8
#define MOV_Wnotfr    0x240  // f5
#define MOV_Wdecfr    0x0C0  // f5
#define MOV_Wincfr    0x280  // f5
#define MOV_Wleftfr   0x340  // f5
#define MOV_Wrightfr  0x300  // f5
#define MOV_Wswapfr   0x380  // f5
#define MOV_WM        0x042  // -
#define MOVSZ_Wdecfr  0x2C0  // f5
#define MOVSZ_Wincfr  0x3C0  // f5
#define MOV_MW        0x043  // -
#define MOV_Mlit      0x050  // k4
#define MOV_rxW       0x000  // f4
#define MOV_OPTIONW   0x002  // -
#define TEST_fr       0x220  // f5

//Program control instructions
#define CALL_ad8 0x900       // k8
#define JMP_ad9	 0xA00       // k9
#define NOP	     0x000       // -
#define RET	     0x00C       // -
#define RETP	 0x00D       // -
#define RETI	 0x00E       // -
#define RETIW	 0x00F       // -
#define RETW_lit 0x800       // k8

//System control instructions
#define BANK_ad8 0x018       // n3
#define IREAD	 0x041       // -
#define PAGE	 0x010       // n3
#define SLEEP	 0x003       // -


#endif
