/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __SYMBOL_H__
#define __SYMBOL_H__

typedef list<int> symbolname;

class symbol
{
  friend int yyparse();
private:
  attribute *attributes;
  int name;
  symbol *parent;
  virtual symbol *_getlocalsymbol(symbolname *) { return (symbol *) NULL; };
  static int nullsymbolcount;

protected:
  list<symbol> *children;
  void addsymbol(symbol *s) { children = new list<symbol>(s, children); }  
  
public:
  friend symbol *nullsymbol(attribute *attribs, symbol *parent);
  symbol(int name, attribute *attributes, symbol *parent);

  virtual void parseattribs(attribute *attribs) {};

  virtual symbolwidget *getwidget();

  attribute *getattribute(int name);
  attribute *getattributes() { return attributes; }

  int getname();
  symbolname *getfullname();
  char *getfullnametext();
  symbol *getparent() { return parent; }
  symbol *getparent(symbolname *s); // Wierd used for variable generation. Avoid at all costs!

  virtual list<symbol> *getapparentsymbols() { return children; }
  symbol *getlocalsymbol(symbolname *s);
  symbol *getsymbol(symbolname *s);
  symbol *getsymbol(char *s);

  // In component creation we need to be able to cast a few types.
  virtual variable *ToVariable() { return (variable *) NULL; }
  virtual pin *ToPin() { return (pin *) NULL; }
  virtual plug *ToPlug() { return (plug *) NULL; }
  virtual component *ToComponent() { return (component *) NULL; }
};

//typedef list<symbol> symbollist;

class symbolconstructor
{
  int defline; // Used to display error messages
  char *DefFile; // Used to display error messages

protected:
  int fullyinitialized;
  int name;
  attribute *attribs;
  virtual symbol *_create(symbol *parent);
  virtual symbol *_ccreate(macrocomponent *parent);

public:
  void addattribstoend(attribute *att);
  void addattribs(attribute *att);
  void initialize(int name, attribute *attribs);
  int getname();
  void ccreate(macrocomponent *parent); // Creates at component creation time. We don't want to create most symbols at this time.
  symbol *create(symbol *parent) ;
  symbolconstructor(int name, attribute *attrib);
  symbolconstructor(attribute *paddingattribute);
  void createerror(int type, char *Msg, ...);
};

class symbolptr : public symbol
{
private:
  symbol *ptr;
  attribute *nameattrib;

  virtual symbol *_getlocalsymbol(symbolname *n) { return ptr->getlocalsymbol(n); };
 
public:
  virtual symbolwidget *getwidget() { symbolwidget *sw = ptr->getwidget(); sw->parseattribs(getattributes()); sw->localparseattribs(nameattrib); return sw; }; //######
  virtual list<symbol> *getapparentsymbols() { return ptr->getapparentsymbols(); };

  virtual variable *ToVariable() { return ptr->ToVariable(); }
  virtual pin *ToPin() { return ptr->ToPin(); }
  virtual plug *ToPlug() { return ptr->ToPlug(); }
  virtual component *ToComponent() { return ptr->ToComponent(); }

  symbolptr(int name, attribute *attribs, symbol *parent, symbol *s);
};
class symbolcopyconstructor : public symbolconstructor
{
  symbolname *orig;
protected:
  virtual symbol *_create(symbol *parent);
public:
  symbolcopyconstructor(int name, attribute *attrib, symbolname *orig);
};

#endif
