/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "project.h"

void fatalerrorexit()
{
  exit(1);
}

void disperrormessage(int type, int line, char *File, char *msg, va_list val)
{
  char *TypeString = "Internal Error in error handler";

  switch (type)
  {
    case TYPE_INTERNALERROR:
      TypeString = "internal error";
      break;
    case TYPE_ERROR:
      TypeString = "error";
      break;
    case TYPE_FATALINTERNALERROR:
      TypeString = "fatal internal error";
      break;
    case TYPE_FATALERROR:
      TypeString = "fatal error";
      break;
    case TYPE_WARNING:
      TypeString = "warning";
      break;
  }

  if (line >= 0) fprintf(stderr, "%s:%i: ", File, line);
  else fprintf(stderr, "?:?:error: ");
  vfprintf(stderr, msg, val);
  fprintf(stderr, "\n");
}

gint Simulate(gpointer data)
{
  Project.IdleLoop();

  if (RunMode == RUN_STOP)
  {
    idletype = 0;
    return FALSE;
  }

  return TRUE;
}

int main(int argc, char ** argv)
{
  FILE *parsein = stdin;
  char *FileName;

  if (argc >= 2) 
  {
    parsein = fopen(argv[1],"r");
    FileName = argv[1];
    if (parsein == NULL) error(TYPE_FATALERROR, -1, NULL, "Error opening file %s", FileName);
  }
  else FileName = "standard input";

  RegisterComponents();
  parse(parsein, FileName);

  gtk_init (&argc, &argv);

  Project.StartInterface();

  //  gtk_idle_add(Simulate, NULL);
  //  gtk_timeout_add(0, Simulate, NULL);
  gtk_main();
}

