/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __PROJECT_H__
#include "project.h"
#endif

#define TARGET_MOVE 1
#define TARGET_HBOX 2
#define TARGET_VBOX 3

struct GtkTargetEntry WidgetTarget = { "symbolwidget", GTK_TARGET_SAME_APP, TARGET_MOVE}; 
struct GtkTargetEntry NewHBoxTarget = { "newhboxwidget", GTK_TARGET_SAME_APP, TARGET_HBOX}; 
struct GtkTargetEntry NewVBoxTarget = { "newvboxwidget", GTK_TARGET_SAME_APP, TARGET_VBOX}; 
struct GtkTargetEntry AllTargets[] = { WidgetTarget, NewHBoxTarget, NewVBoxTarget };

/****************************************************************************

			       WIDGET

 ****************************************************************************/

widget::~widget()
{
  // #### Err!!
}

void widget::display()
{
  if (startup) 
  {
    parseattribs(initattribs);
    startup = 0;
  }
  draw();
  passtochildren(&symbolwidget::display);
  gtk_widget_show(wi);
}

void widget::refresh()
{
  draw();
  passtochildren(&symbolwidget::refresh);
}

widget::widget(attribute *attribs) 
{ 
  wi = (GtkWidget *) NULL; 
  initattribs = attribs;
  startup = 1;
}

void widget::draw()
{ 
}

void widget::passtochildren(void (widget::*fn)())
{
}

void widget::RemoveFromContainer(GtkWidget *c)
{ 
  gtk_container_remove(GTK_CONTAINER(c), wi); 
}

/****************************************************************************

				 WINDOW

 ****************************************************************************/

void window::RemoveChild(symbolborderwidget *sbw)
{
  if (sbw != w) { cerr << "Internal error : window::RemoveChild on non parent window."; exit(1); }

  sbw->RemoveFromContainer(this->box);

  w = NULL;
  if (!destroying) delete this;
}

window::~window()
{
  if (destroying)
  {
    cerr << "Internal error : window::~window reentered.";
    exit(1);
  }

  destroying = 1; // Avoid recursive calls to the destructor
  //  gtk_signal_disconnect_by_func(GTK_OBJECT(wi), GTK_SIGNAL_FUNC(window::delete_event), this); // Avoid having the distructor called when we delete the GTK window...

  cout << "Win destruct\n";
  Project.UnlinkWindow(this);
  cout << "Win destruct2\n";
  delete w; 
  gtk_widget_hide(wi);
  gtk_widget_destroy(wi);
}

void window::passtochildren(void (widget::*fn)())
{
  (w->*fn)(); 
}

void window::delete_event(GtkWidget *widget, GdkEvent  *event, window *_this)
{
  if (!_this->destroying) delete _this;
  else cout << "Totobabamimi\n";
}

void window::newwindow(GtkButton *button, int type)
{
  window *newwin = new window(Project.getattributes(), displaybox::getempty(type)->getwidget());
  newwin->display();
}

void window::trashusedropdata(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time,  gpointer udata)
{
  symbolborderwidget *symb = *(symbolborderwidget **) data->data;
  cerr << "\nyyyi" << symb;
  gtk_drag_finish(drag_context, TRUE, TRUE, time);
  
  delete symb;
}

void window::getboxdata(GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *data, guint info, guint time, window *_this)
{
  static gchar dummy;
  cerr << "\ntoto2" << _this;
  gtk_selection_data_set(data, 0, 0, NULL, 0);
}

window::window(attribute *attribs, symbolwidget *sw) : widget(attribs)
{
  destroying = 0;
  GtkWidget *toolbox;
  toolbox = gtk_hbox_new(TRUE, 0);

  GtkWidget *CurButton;

  CurButton = gtk_button_new_with_label("New Window\nHorizontal");
  gtk_signal_connect(GTK_OBJECT(CurButton), "clicked", GTK_SIGNAL_FUNC(window::newwindow), BOX_H);
  gtk_widget_show(CurButton);
  gtk_box_pack_start_defaults(GTK_BOX(toolbox), CurButton);

  CurButton = gtk_button_new_with_label("New Window\nVertical");
  gtk_signal_connect(GTK_OBJECT(CurButton), "clicked", GTK_SIGNAL_FUNC(window::newwindow), BOX_V);
  gtk_widget_show(CurButton);
  gtk_box_pack_start_defaults(GTK_BOX(toolbox), CurButton);

  CurButton = gtk_button_new_with_label("New Box\nHorizontal");
  gtk_drag_source_set(CurButton, GDK_BUTTON1_MASK, &NewHBoxTarget, 1, GDK_ACTION_MOVE);
  gtk_signal_connect(GTK_OBJECT (CurButton), "drag-data-get", GTK_SIGNAL_FUNC(window::getboxdata), this);
  gtk_widget_show(CurButton);
  gtk_box_pack_start_defaults(GTK_BOX(toolbox), CurButton);

  CurButton = gtk_button_new_with_label("New Box\nVertical");
  gtk_drag_source_set(CurButton, GDK_BUTTON1_MASK, &NewVBoxTarget, 1, GDK_ACTION_MOVE);
  gtk_signal_connect(GTK_OBJECT (CurButton), "drag-data-get", GTK_SIGNAL_FUNC(window::getboxdata), this);
  gtk_widget_show(CurButton);
  gtk_box_pack_start_defaults(GTK_BOX(toolbox), CurButton);

  CurButton = gtk_button_new_with_label("Trash\nCan");
  gtk_drag_dest_set(CurButton, GTK_DEST_DEFAULT_ALL, &WidgetTarget, 1, GDK_ACTION_MOVE);
  gtk_signal_connect(GTK_OBJECT(CurButton), "drag-data-received", GTK_SIGNAL_FUNC(window::trashusedropdata), this);
  gtk_widget_show(CurButton);
  gtk_box_pack_start_defaults(GTK_BOX(toolbox), CurButton);

  // Tool bar

  gtk_widget_show(toolbox);

  // Main window contents
  window::w = new symbolborderwidget(attribs, sw);

  box = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX(box), toolbox);
  gtk_box_pack_start_defaults(GTK_BOX(box), w->GrabGtkWidget(this, (GtkWidget *) NULL));
  gtk_widget_show(box);

  // Main window

  wi = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT (wi), "delete_event", GTK_SIGNAL_FUNC(window::delete_event), this);
  gtk_container_add(GTK_CONTAINER(wi), box);

  Project.windows = new list<window>(this, Project.windows);
  display();
}

void window::draw()
{
  /*  char buff[100];

  snprintf(buff, 100, "Time : %f", Project.GetTime());
  gtk_window_set_title(GTK_WINDOW (wi), buff);*/
}

/****************************************************************************

			SYMBOLBORDERWIDGET

 ****************************************************************************/

symbolborderwidget::~symbolborderwidget()
{
  GrabGtkWidget((widget *) NULL, (GtkWidget *) NULL);
  delete s;
}

void symbolborderwidget::passtochildren(void (widget::*fn)())
{
  (s->*fn)();
}

/*void test2(GtkWidget *w)
{
  gtk_drag_highlight(w);  
}

void test3(GtkWidget *w)
{
  gtk_drag_unhighlight(w);  
  }*/

symbolborderwidget::symbolborderwidget(attribute *attribs, symbolwidget *s) : widget(attribs)
{
  AssociatedSeparator = NULL;
  symbolborderwidget::s = s;
  displaymode.title = !symbtable.isanonymous(s->getsymbol()->getname());
  displaymode.full = 0;
  displaymode.hasborder = s->hasborder();
  parent = 0;

  if (!displaymode.hasborder)
  {
    title = gtk_label_new(NULL);
    settitle();
    gtk_widget_show(title);

    border = gtk_hbox_new(FALSE, 0);
    gtk_box_set_spacing(GTK_BOX(border), 10);
    gtk_box_pack_start_defaults(GTK_BOX(border), s->GetGtkWidget(this));
    gtk_box_pack_start_defaults(GTK_BOX(border), title);
  }
  else
  {
    border = gtk_frame_new(NULL);
    gtk_container_set_border_width(GTK_CONTAINER(border), 5);
    settitle();
    gtk_container_add(GTK_CONTAINER(border), s->GetGtkWidget(this));
  }

  gtk_widget_show(border);
  wi = gtk_event_box_new();
  gtk_container_add(GTK_CONTAINER(wi), border);
  gtk_drag_source_set(wi, GDK_BUTTON1_MASK, &WidgetTarget, 1, GDK_ACTION_MOVE);
  gtk_signal_connect(GTK_OBJECT (wi), "drag-begin", GTK_SIGNAL_FUNC(symbolborderwidget::begindrag), this);
  gtk_signal_connect(GTK_OBJECT (wi), "drag-end", GTK_SIGNAL_FUNC(symbolborderwidget::enddrag), this);
  gtk_signal_connect(GTK_OBJECT (wi), "drag-data-get", GTK_SIGNAL_FUNC(symbolborderwidget::getdropdata), this);
  gtk_signal_connect(GTK_OBJECT (wi), "enter-notify-event", GTK_SIGNAL_FUNC(gtk_drag_highlight), this);
  gtk_signal_connect(GTK_OBJECT (wi), "leave-notify-event", GTK_SIGNAL_FUNC(gtk_drag_unhighlight), this);
}

void symbolborderwidget::symbolborderwidget::begindrag(GtkWidget *widget, GdkDragContext *drag_context, symbolborderwidget *_this)
{
  cerr << "a";
  if (_this->parent->GetWindowWidget() != NULL) gtk_widget_hide(_this->parent->GetWindowWidget());
  gtk_widget_hide(_this->wi);
  if (_this->AssociatedSeparator != NULL) gtk_widget_hide(_this->AssociatedSeparator);
}

void symbolborderwidget::enddrag(GtkWidget *widget, GdkDragContext *drag_context, symbolborderwidget *_this)
{
  cerr << "b";
  if (drag_context -> action == 0)
  {
    cerr << "o";
    gtk_widget_show(_this->wi);
    if (_this->AssociatedSeparator != NULL) gtk_widget_show(_this->AssociatedSeparator);
    if (_this->parent->GetWindowWidget() != NULL) gtk_widget_show(_this->parent->GetWindowWidget());
  }
  cerr << "c";
}

void symbolborderwidget::getdropdata(GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *data, guint info, guint time, symbolborderwidget *_this)
{
  cerr << "\ntoto" << _this;
  gtk_selection_data_set(data, 0, sizeof(symbolborderwidget *) * 8, (guchar *) &_this, sizeof(symbolborderwidget *));
}

void symbolborderwidget::settitle()
{
  char *txt;
 
  if (!displaymode.title) return;//txt = (char *) NULL;
  else txt = displaymode.full ? s->getsymbol()->getfullnametext() : symbtable.tostring(s->getsymbol()->getname());
  
  if (displaymode.hasborder) gtk_frame_set_label(GTK_FRAME(border), txt);
  else gtk_label_set_text(GTK_LABEL(border), txt);
}

GtkWidget *symbolborderwidget::GrabGtkWidget(widget *p, GtkWidget *AssocSep) 
{ 
  if (parent != NULL) 
  {
    parent->RemoveChild(this);
  }

  AssociatedSeparator = AssocSep;
  parent = p; 
  return wi; 
}

/****************************************************************************

				   SYMBOLWIDGET

 ****************************************************************************/

symbolwidget::symbolwidget(attribute *attribs, symbol *s) : widget(attribs)
{
  symbolwidget::s = s;
}


/****************************************************************************

				   PLAINSYMBOLWIDGET

 ****************************************************************************/

plainsymbolwidget::plainsymbolwidget(attribute *attribs, symbol *s) : symbolwidget(attribs, s)
{
  char buffer[100];
  snprintf(buffer, 100, "Symbol without its widget type :\n%s", s->getfullnametext());
  wi = gtk_label_new(buffer);
}

/****************************************************************************

				   PINWIDGET

 ****************************************************************************/

pinwidget::pinwidget(attribute *attribs, pin *p) : symbolwidget(attribs, p)
{
  wi = gtk_label_new("");
}

void pinwidget::draw()
{
  char buff[100];

  pin *p = (pin *) getsymbol();

  snprintf(buff, 100, "Vin(%f) Vout(%f) Z(%f)", p->GetVin(), p->GetVout(), p->GetZ());
  gtk_label_set_text(GTK_LABEL(wi), buff);
}

/****************************************************************************

				   BOXWIDGET

 ****************************************************************************/

void boxwidget::passtochildren(void (widget::*fn)())
{
  for (list <symbolborderwidget> *l = widgets; l != NULL; l = l->next)
    (l->elem->*fn)();
}

boxwidget::boxwidget(attribute *attribs, displaybox *boxs, GtkBox *box, GtkWidget *(*new_separator_func)()) : symbolwidget(attribs, boxs)
{
  list<symbolwidget> *contents = (list<symbolwidget> *) NULL;
  int tnameval;
  list<int> tname(&tnameval, (list<int> *) NULL);
  symbol *s;
  
  for (list<symbolconstructor> *l = boxs->constr->constructors; l != NULL; l = l->next)
  {
    tnameval = l->elem->getname();
    s = boxs->getlocalsymbol(&tname);
    if (s == NULL) boxs->constr->createerror("Can't find %s to create box.", symbtable.tostring(tnameval));
    contents = new list<symbolwidget>(s->getwidget(), contents);
  }

  // Now that we have the symbolwidgets, make the symbolborderwidgets
  // #### Memory leak
  widgets = (list<symbolborderwidget> *) NULL;
  for (;contents != NULL; contents = contents->next)
    widgets = new list<symbolborderwidget>(new symbolborderwidget(attribs, contents->elem), widgets);

  // Time to create the various widgets
  boxwidget::new_separator_func = new_separator_func;

  wi = GTK_WIDGET(box);
  //  gtk_container_set_border_width(GTK_CONTAINER(wi), 5);
  
  lastseparator = newseparator();
  for (list<symbolborderwidget> *t = widgets; t != NULL; t = t->next)
  {
    gtk_box_pack_start_defaults(box, t->elem->GrabGtkWidget(this, lastseparator));
    lastseparator = newseparator();
  }
}

/*void test()
{
  cerr << "hah";
}
*/

GtkWidget *boxwidget::newseparator()
{
  GtkWidget *w = new_separator_func();
  gtk_widget_set_usize(w, 7, 7);
  gtk_box_pack_start_defaults(GTK_BOX(wi), w);
  gtk_widget_show(w);

// Start drag and drop
  gtk_drag_dest_set(w, GTK_DEST_DEFAULT_ALL, AllTargets, 3, GDK_ACTION_MOVE);

  gtk_signal_connect(GTK_OBJECT (w), "drag-data-received", GTK_SIGNAL_FUNC(boxwidget::usedropdata), this);
  //  gtk_signal_connect(GTK_OBJECT (w), "drag-motion", GTK_SIGNAL_FUNC(test), this);

  return w;
}

void boxwidget::usedropdata(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time, boxwidget *_this)
{
  symbolborderwidget *symb;
  
  switch (info)
  {
    case TARGET_MOVE:
      symb= *(symbolborderwidget **) data->data;
      gtk_drag_finish(drag_context, TRUE, TRUE, time);
      break;
    case TARGET_HBOX:
      symb = new symbolborderwidget(_this->initattribs, displaybox::getempty(BOX_H)->getwidget());
      break;
    case TARGET_VBOX:
      symb = new symbolborderwidget(_this->initattribs, displaybox::getempty(BOX_V)->getwidget());
      break;
    default:
      cout << "Internal error : Bad Target type in drag and drop!";
      exit(1);
  }

  cerr << "\nyyyi" << symb;

  _this->insertwidget(symb, widget);
  //  gtk_widget_hide_all(gtk_widget_get_toplevel(widget));
  //  gtk_widget_show_all(gtk_widget_get_toplevel(widget));
}

void boxwidget::insertwidget(symbolborderwidget *symb, GtkWidget *psep)
{
  list<symbolborderwidget> *l;

   // Have we moved?
  int wasprev = 0;
  for (l = widgets; l != NULL; l = l->next)
  {
    if (l->elem->GetAssociatedSeparator() == psep) break;
    wasprev = 0;
    if (l->elem->GetAssociatedSeparator() == symb->GetAssociatedSeparator()) wasprev = 1;
  }
  if (wasprev) 
    if (l != NULL || lastseparator == psep)
    {
      symb->display();
      if (symb->GetAssociatedSeparator() != NULL) gtk_widget_show(symb->GetAssociatedSeparator());
      if (symb->getparent()->GetWindowWidget() != NULL) gtk_widget_show(symb->getparent()->GetWindowWidget());
      return; // Not moved
    }

  // This removes symbol from its parent widget.
  GtkWidget *newsep = newseparator();
  GtkWidget *child = symb->GrabGtkWidget(this, newsep);

  // Now we actually insert
  int pos = 0;
  list<symbolborderwidget> **pl = &widgets;
  for (l = widgets; l != NULL; l = *pl, pos++)
  {
    if (l->elem->GetAssociatedSeparator() == psep)
      break;
    pl = &(l->next);
  }

  if (l == NULL && lastseparator != psep) 
  {
    cout << "Internal error : boxwidget::insertwidget called with invalid separator\n";
    exit(1);
  }

  *pl = new list<symbolborderwidget>(symb, l);
  
  gtk_box_pack_start_defaults(GTK_BOX(this->wi), child);
  gtk_box_reorder_child(GTK_BOX(this->wi), newsep, pos * 2 + 0);
  gtk_box_reorder_child(GTK_BOX(this->wi), child, pos * 2 + 1);

  symb->display();
  if (symb->GetAssociatedSeparator() != NULL) gtk_widget_show(symb->GetAssociatedSeparator());
  if (symb->getparent()->GetWindowWidget() != NULL) gtk_widget_show(symb->getparent()->GetWindowWidget());
}

void boxwidget::RemoveChild(symbolborderwidget *sbw)
{
  list<symbolborderwidget> **pw = &widgets;
  for (list<symbolborderwidget> *w = widgets; w != NULL; w = *pw)
  {
    if (w->elem == sbw)
    {
      sbw->RemoveFromContainer(this->wi);
      gtk_widget_hide(sbw->GetAssociatedSeparator());
      gtk_widget_destroy(sbw->GetAssociatedSeparator());
      *pw = w->next;
      return;
    }
    pw = &(w->next);
  }

  cout << "Internal error : boxwidget::RemoveChild called on non child\n"; 
  exit(1);
}

hboxwidget::hboxwidget(attribute *attribs, displaybox *s) : boxwidget(attribs, s,GTK_BOX(gtk_hbox_new(FALSE, 0)), gtk_vseparator_new)
{
}

vboxwidget::vboxwidget(attribute *attribs, displaybox *s) : boxwidget(attribs, s,GTK_BOX(gtk_vbox_new(FALSE, 0)), gtk_hseparator_new)
{
}

/*consttextwidget::consttextwidget(attribute *attribs, char *text) : symbolwidget(attribs)
{
  wi = gtk_label_new(text);
  // wi = gtk_entry_new();
  // gtk_entry_set_text(GTK_ENTRY(wi), text);
}
*/

/****************************************************************************

				   VARIABLEWIDGET

 ****************************************************************************/

variablewidget::variablewidget(attribute *attribs, variable *var) : symbolwidget(attribs, var)
{
  wi = gtk_label_new("");
}

void variablewidget::draw()
{
  char buff[100];

  variable *var = (variable *) getsymbol();

  snprintf(buff, 100, "%li", var->getvalue());
  gtk_label_set_text(GTK_LABEL(wi), buff);
}

/****************************************************************************

				   PLUGWIDGET

 ****************************************************************************/

plugwidget::plugwidget(attribute *attribs, plug *p) : vboxwidget(attribs, displaybox::getempty(BOX_V))
{
  for (list<symbol> *l = p->symbols; l != NULL; l = l->next)
  {
    insertwidget(new symbolborderwidget(attribs, l->elem->getwidget()), lastseparator);
  }
}
